/*
 * Copyright (c) 2010-2011, The MiCode Open Source Community (www.micode.net)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.micode.notes.gtask.data;

import android.content.ContentResolver;
import android.content.ContentUris;
import android.content.ContentValues;
import android.content.Context;
import android.database.Cursor;
import android.net.Uri;
import android.util.Log;

import net.micode.notes.data.Notes;
import net.micode.notes.data.Notes.DataColumns;
import net.micode.notes.data.Notes.DataConstants;
import net.micode.notes.data.Notes.NoteColumns;
import net.micode.notes.data.NotesDatabaseHelper.TABLE;
import net.micode.notes.gtask.exception.ActionFailureException;

import org.json.JSONException;
import org.json.JSONObject;


public class SqlData {
    // 定义日志标签，用于日志输出
    private static final String TAG = SqlData.class.getSimpleName();

    // 定义无效ID常量
    private static final int INVALID_ID = -99999;

    // 定义查询数据时使用的列名数组
    public static final String[] PROJECTION_DATA = new String[] {
            DataColumns.ID, DataColumns.MIME_TYPE, DataColumns.CONTENT, DataColumns.DATA1,
            DataColumns.DATA3
    };

    // 定义各列在PROJECTION_DATA数组中的索引
    public static final int DATA_ID_COLUMN = 0;

    public static final int DATA_MIME_TYPE_COLUMN = 1;

    public static final int DATA_CONTENT_COLUMN = 2;

    public static final int DATA_CONTENT_DATA_1_COLUMN = 3;

    public static final int DATA_CONTENT_DATA_3_COLUMN = 4;

    // 内容解析器，用于操作数据库
    private ContentResolver mContentResolver;

    // 标记是否为创建操作
    private boolean mIsCreate;

    // 数据ID
    private long mDataId;

    // 数据的MIME类型
    private String mDataMimeType;

    // 数据内容
    private String mDataContent;

    // 数据内容中的第一个附加数据
    private long mDataContentData1;

    // 数据内容中的第三个附加数据
    private String mDataContentData3;

    // 存储与原始数据不同的数据值
    private ContentValues mDiffDataValues;

    // 构造函数，用于初始化SqlData对象
    public SqlData(Context context) {
        mContentResolver = context.getContentResolver();
        mIsCreate = true;
        mDataId = INVALID_ID;
        mDataMimeType = DataConstants.NOTE;
        mDataContent = "";
        mDataContentData1 = 0;
        mDataContentData3 = "";
        mDiffDataValues = new ContentValues();
    }

    // 构造函数，用于从Cursor对象中加载数据
    public SqlData(Context context, Cursor c) {
        mContentResolver = context.getContentResolver();
        mIsCreate = false;
        loadFromCursor(c);
        mDiffDataValues = new ContentValues();
    }

    // 定义一个私有方法，用于从Cursor对象中加载数据
    private void loadFromCursor(Cursor c) {
        // 从Cursor对象中获取指定列的数据，并将其赋值给成员变量mDataId
        mDataId = c.getLong(DATA_ID_COLUMN);
        // 从Cursor对象中获取指定列的数据，并将其赋值给成员变量mDataMimeType
        mDataMimeType = c.getString(DATA_MIME_TYPE_COLUMN);
        // 从Cursor对象中获取指定列的数据，并将其赋值给成员变量mDataContent
        mDataContent = c.getString(DATA_CONTENT_COLUMN);
        // 从Cursor对象中获取指定列的数据，并将其赋值给成员变量mDataContentData1
        mDataContentData1 = c.getLong(DATA_CONTENT_DATA_1_COLUMN);
        // 从Cursor对象中获取指定列的数据，并将其赋值给成员变量mDataContentData3
        mDataContentData3 = c.getString(DATA_CONTENT_DATA_3_COLUMN);
    }

    // 设置数据内容，根据传入的JSONObject更新成员变量和差异数据值
    public void setContent(JSONObject js) throws JSONException {
        long dataId = js.has(DataColumns.ID) ? js.getLong(DataColumns.ID) : INVALID_ID;
        if (mIsCreate || mDataId != dataId) {
            mDiffDataValues.put(DataColumns.ID, dataId);
        }
        mDataId = dataId;

        String dataMimeType = js.has(DataColumns.MIME_TYPE) ? js.getString(DataColumns.MIME_TYPE)
                : DataConstants.NOTE;
        if (mIsCreate || !mDataMimeType.equals(dataMimeType)) {
            mDiffDataValues.put(DataColumns.MIME_TYPE, dataMimeType);
        }
        mDataMimeType = dataMimeType;

        String dataContent = js.has(DataColumns.CONTENT) ? js.getString(DataColumns.CONTENT) : "";
        if (mIsCreate || !mDataContent.equals(dataContent)) {
            mDiffDataValues.put(DataColumns.CONTENT, dataContent);
        }
        mDataContent = dataContent;

        long dataContentData1 = js.has(DataColumns.DATA1) ? js.getLong(DataColumns.DATA1) : 0;
        if (mIsCreate || mDataContentData1 != dataContentData1) {
            mDiffDataValues.put(DataColumns.DATA1, dataContentData1);
        }
        mDataContentData1 = dataContentData1;

        String dataContentData3 = js.has(DataColumns.DATA3) ? js.getString(DataColumns.DATA3) : "";
        if (mIsCreate || !mDataContentData3.equals(dataContentData3)) {
            mDiffDataValues.put(DataColumns.DATA3, dataContentData3);
        }
        mDataContentData3 = dataContentData3;
    }

    // 获取数据内容的JSONObject表示
    public JSONObject getContent() throws JSONException {
        if (mIsCreate) {
            Log.e(TAG, "it seems that we haven't created this in database yet");
            return null;
        }
        JSONObject js = new JSONObject();
        js.put(DataColumns.ID, mDataId);
        js.put(DataColumns.MIME_TYPE, mDataMimeType);
        js.put(DataColumns.CONTENT, mDataContent);
        js.put(DataColumns.DATA1, mDataContentData1);
        js.put(DataColumns.DATA3, mDataContentData3);
        return js;
    }

    // 提交数据到数据库
    public void commit(long noteId, boolean validateVersion, long version) {

        // 检查是否是创建新笔记的操作
        if (mIsCreate) {
            // 如果数据ID无效且差异数据值中包含ID字段，则移除ID字段
            if (mDataId == INVALID_ID && mDiffDataValues.containsKey(DataColumns.ID)) {
                mDiffDataValues.remove(DataColumns.ID);
            }

            // 将笔记ID添加到差异数据值中
            mDiffDataValues.put(DataColumns.NOTE_ID, noteId);
            // 使用内容解析器插入新的笔记数据
            Uri uri = mContentResolver.insert(Notes.CONTENT_DATA_URI, mDiffDataValues);
            try {
                // 从返回的URI中提取笔记ID
                mDataId = Long.valueOf(uri.getPathSegments().get(1));
            } catch (NumberFormatException e) {
                // 如果解析笔记ID失败，记录错误并抛出异常
                Log.e(TAG, "Get note id error :" + e.toString());
                throw new ActionFailureException("create note failed");
            }
        } else {
            // 如果不是创建操作，且差异数据值不为空
            if (mDiffDataValues.size() > 0) {
                int result = 0;
                // 根据是否需要验证版本号来更新笔记数据
                if (!validateVersion) {
                    // 不验证版本号直接更新
                    result = mContentResolver.update(ContentUris.withAppendedId(
                            Notes.CONTENT_DATA_URI, mDataId), mDiffDataValues, null, null);
                } else {
                    // 验证版本号后更新
                    result = mContentResolver.update(ContentUris.withAppendedId(
                            Notes.CONTENT_DATA_URI, mDataId), mDiffDataValues,
                            " ? in (SELECT " + NoteColumns.ID + " FROM " + TABLE.NOTE
                                    + " WHERE " + NoteColumns.VERSION + "=?)", new String[] {
                                    String.valueOf(noteId), String.valueOf(version)
                            });
                }
                // 如果更新结果为0，表示没有更新成功，记录警告日志
                if (result == 0) {
                    Log.w(TAG, "there is no update. maybe user updates note when syncing");
                }
            }
        }

        // 清空差异数据值
        mDiffDataValues.clear();
        // 将创建标志设置为false
        mIsCreate = false;
    }

    // 获取ID
    public long getId() {
        return mDataId;
    }
}
