/*
 * Copyright (c) 2010-2011, The MiCode Open Source Community (www.micode.net)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.micode.notes.ui;

import android.app.Activity;
import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.DialogInterface.OnClickListener;
import android.content.DialogInterface.OnDismissListener;
import android.content.Intent;
import android.media.AudioManager;
import android.media.MediaPlayer;
import android.media.RingtoneManager;
import android.net.Uri;
import android.os.Bundle;
import android.os.PowerManager;
import android.provider.Settings;
import android.view.Window;
import android.view.WindowManager;

import net.micode.notes.R;
import net.micode.notes.data.Notes;
import net.micode.notes.tool.DataUtils;

import java.io.IOException;


public class AlarmAlertActivity extends Activity implements OnClickListener, OnDismissListener {
    private long mNoteId;
    private String mSnippet;
    private static final int SNIPPET_PREW_MAX_LEN = 60;
    MediaPlayer mPlayer;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        // 调用父类的onCreate方法，初始化Activity
        super.onCreate(savedInstanceState);
        // 请求无标题窗口特性
        requestWindowFeature(Window.FEATURE_NO_TITLE);

        // 获取当前Activity的Window对象
        final Window win = getWindow();
        // 设置窗口标志，使窗口在锁屏时仍然可见
        win.addFlags(WindowManager.LayoutParams.FLAG_SHOW_WHEN_LOCKED);

        // 检查屏幕是否处于关闭状态
        if (!isScreenOn()) {
            // 如果屏幕关闭，则添加多个标志以保持屏幕常亮、解锁屏幕并允许在屏幕上显示内容
            win.addFlags(WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON
                    | WindowManager.LayoutParams.FLAG_TURN_SCREEN_ON
                    | WindowManager.LayoutParams.FLAG_ALLOW_LOCK_WHILE_SCREEN_ON
                    | WindowManager.LayoutParams.FLAG_LAYOUT_INSET_DECOR);
        }

        // 获取当前Activity的Intent对象
        Intent intent = getIntent();

        try {
            // 从Intent的数据中获取Note的ID
            mNoteId = Long.valueOf(intent.getData().getPathSegments().get(1));
            // 根据Note的ID从内容解析器中获取对应的Snippet
            mSnippet = DataUtils.getSnippetById(this.getContentResolver(), mNoteId);
            // 如果Snippet长度超过预定义的最大长度，则截取并添加提示信息
            mSnippet = mSnippet.length() > SNIPPET_PREW_MAX_LEN ? mSnippet.substring(0,
                    SNIPPET_PREW_MAX_LEN) + getResources().getString(R.string.notelist_string_info)
                    : mSnippet;
        } catch (IllegalArgumentException e) {
            // 捕获并打印IllegalArgumentException异常
            e.printStackTrace();
            // 发生异常时，直接返回，不继续执行后续代码
            return;
        }

        // 创建一个新的MediaPlayer对象
        mPlayer = new MediaPlayer();
        // 检查Note是否在Note数据库中可见
        if (DataUtils.visibleInNoteDatabase(getContentResolver(), mNoteId, Notes.TYPE_NOTE)) {
            // 如果可见，则显示操作对话框并播放警报声音
            showActionDialog();
            playAlarmSound();
        } else {
            // 如果不可见，则结束当前Activity
            finish();
        }
    }

    // 定义一个私有方法isScreenOn，用于检查设备屏幕是否处于开启状态
    private boolean isScreenOn() {
        // 获取PowerManager系统服务，用于管理电源相关的功能
        PowerManager pm = (PowerManager) getSystemService(Context.POWER_SERVICE);
        // 调用PowerManager的isScreenOn方法，返回屏幕是否开启的布尔值
        return pm.isScreenOn();
    }

    // 定义一个私有方法，用于播放警报声音
    private void playAlarmSound() {
        // 获取系统默认的警报铃声Uri
        Uri url = RingtoneManager.getActualDefaultRingtoneUri(this, RingtoneManager.TYPE_ALARM);

        // 获取当前系统设置中静音模式影响的音频流类型
        int silentModeStreams = Settings.System.getInt(getContentResolver(),
                Settings.System.MODE_RINGER_STREAMS_AFFECTED, 0);

        // 检查当前系统是否将警报音频流类型包含在静音模式中
        if ((silentModeStreams & (1 << AudioManager.STREAM_ALARM)) != 0) {
            // 如果包含，则设置MediaPlayer的音频流类型为静音模式影响的类型
            mPlayer.setAudioStreamType(silentModeStreams);
        } else {
            // 如果不包含，则设置MediaPlayer的音频流类型为警报类型
            mPlayer.setAudioStreamType(AudioManager.STREAM_ALARM);
        }
        try {
            // 设置MediaPlayer的数据源为警报铃声的Uri
            mPlayer.setDataSource(this, url);
            // 准备MediaPlayer
            mPlayer.prepare();
            // 设置MediaPlayer循环播放
            mPlayer.setLooping(true);
            // 开始播放
            mPlayer.start();
        } catch (IllegalArgumentException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        } catch (SecurityException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        } catch (IllegalStateException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        } catch (IOException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
    }

    // 定义一个私有方法showActionDialog，用于显示一个操作对话框
    private void showActionDialog() {
        // 创建一个AlertDialog.Builder对象，用于构建对话框
        AlertDialog.Builder dialog = new AlertDialog.Builder(this);
        // 设置对话框的标题，使用资源文件中的字符串
        dialog.setTitle(R.string.app_name);
        // 设置对话框的消息内容，使用成员变量mSnippet的值
        dialog.setMessage(mSnippet);
        // 设置对话框的确定按钮，并指定点击事件监听器为当前对象（this）
        dialog.setPositiveButton(R.string.notealert_ok, this);
        // 检查屏幕是否处于开启状态
        if (isScreenOn()) {
            // 如果屏幕开启，则设置对话框的取消按钮，并指定点击事件监听器为当前对象（this）
            dialog.setNegativeButton(R.string.notealert_enter, this);
        }
        // 显示对话框，并设置对话框关闭时的监听器为当前对象（this）
        dialog.show().setOnDismissListener(this);
    }

    // 定义一个方法来处理对话框点击事件
    public void onClick(DialogInterface dialog, int which) {
        // 使用switch语句根据点击的按钮类型进行不同的处理
        switch (which) {
            // 如果点击的是否定按钮（例如“取消”或“否”）
            case DialogInterface.BUTTON_NEGATIVE:
                // 创建一个新的Intent对象，用于启动NoteEditActivity
                Intent intent = new Intent(this, NoteEditActivity.class);
                // 设置Intent的动作类型为查看（ACTION_VIEW）
                intent.setAction(Intent.ACTION_VIEW);
                // 向Intent中添加一个额外的数据，即笔记的ID
                intent.putExtra(Intent.EXTRA_UID, mNoteId);
                // 启动NoteEditActivity，传递Intent对象
                startActivity(intent);
                break;
            // 默认情况下，不进行任何操作
            default:
                break;
        }
    }

    public void onDismiss(DialogInterface dialog) {
        stopAlarmSound();
        finish();
    }

    private void stopAlarmSound() {
        if (mPlayer != null) {
            mPlayer.stop();
            mPlayer.release();
            mPlayer = null;
        }
    }
}
